@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cpack.cxx,v $
   Language:  C++
-  Date:      $Date: 2008-04-21 00:44:53 $
-  Version:   $Revision: 1.42.2.1 $
+  Date:      $Date: 2009-02-04 16:44:18 $
+  Version:   $Revision: 1.42.2.3 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -382,7 +382,7 @@ int main (int argc, char *argv[])
               << gen << std::endl);
             parsed = 0;
             }
-          if ( parsed && !cpackGenerator->Initialize(gen, mf, argv[0]) )
+          if ( parsed && !cpackGenerator->Initialize(gen, mf) )
             {
             cmCPack_Log(&log, cmCPackLog::LOG_ERROR,
               "Cannot initialize the generator " << gen << std::endl);
@@ -474,5 +474,10 @@ int main (int argc, char *argv[])
 #define cout no_cout_use_cmCPack_Log
     }
 
+  if (cmSystemTools::GetErrorOccuredFlag())
+    {
+    return 1;
+    }
+
   return 0;
 }
