@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmAddCustomTargetCommand.h,v $
   Language:  C++
-  Date:      $Date: 2006/06/30 17:48:43 $
-  Version:   $Revision: 1.15.2.1 $
+  Date:      $Date: 2008-10-24 15:18:45 $
+  Version:   $Revision: 1.22.2.1 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -41,13 +41,14 @@ public:
    * This is called when the command is first encountered in
    * the CMakeLists.txt file.
    */
-  virtual bool InitialPass(std::vector<std::string> const& args);
+  virtual bool InitialPass(std::vector<std::string> const& args,
+                           cmExecutionStatus &status);
   
   /**
    * The name of the command as specified in CMakeList.txt.
    */
   virtual const char* GetName() 
-    {return "ADD_CUSTOM_TARGET";}
+    {return "add_custom_target";}
   
   /**
    * Succinct documentation.
@@ -63,12 +64,19 @@ public:
   virtual const char* GetFullDocumentation()
     {
     return
-      "  ADD_CUSTOM_TARGET(Name [ALL] [command1 [args1...]]\n"
+      "  add_custom_target(Name [ALL] [command1 [args1...]]\n"
       "                    [COMMAND command2 [args2...] ...]\n"
-      "                    [DEPENDS depend depend depend ... ])\n"
+      "                    [DEPENDS depend depend depend ... ]\n"
       "                    [WORKING_DIRECTORY dir]\n"
-      "Adds a target with the given name that executes the given commands "
-      "every time the target is built.  If the ALL option is specified "
+      "                    [COMMENT comment] [VERBATIM]\n"
+      "                    [SOURCES src1 [src2...]])\n"
+      "Adds a target with the given name that executes the given commands. "
+      "The target has no output file and is ALWAYS CONSIDERED OUT OF DATE "
+      "even if the commands try to create a file with the name of the "
+      "target. Use ADD_CUSTOM_COMMAND to generate a file with dependencies. "
+      "By default nothing depends on the custom target. Use "
+      "ADD_DEPENDENCIES to add dependencies to or from other targets. "
+      "If the ALL option is specified "
       "it indicates that this target should be added to the default build "
       "target so that it will be run every time "
       "(the command cannot be called ALL). "
@@ -76,10 +84,24 @@ public:
       "empty target will be created. "
       "If WORKING_DIRECTORY is set, then the command will be run in that "
       "directory. "
+      "If COMMENT is set, the value will be displayed as a "
+      "message before the commands are executed at build time. "
       "Dependencies listed with the DEPENDS argument may reference files "
-      "and outputs of custom commands created with ADD_CUSTOM_COMMAND. "
-      "Dependencies on other targets may be added using the "
-      "ADD_DEPENDENCIES command.";
+      "and outputs of custom commands created with ADD_CUSTOM_COMMAND.\n"
+      "If VERBATIM is given then all the arguments to the commands will be "
+      "passed exactly as specified no matter the build tool used. "
+      "Note that one level of escapes is still used by the CMake language "
+      "processor before add_custom_target even sees the arguments. "
+      "Use of VERBATIM is recommended as it enables correct behavior. "
+      "When VERBATIM is not given the behavior is platform specific. "
+      "In the future VERBATIM may be enabled by default. The only reason "
+      "it is an option is to preserve compatibility with older CMake code."
+      "\n"
+      "The SOURCES option specifies additional source files to be included "
+      "in the custom target.  "
+      "Specified source files will be added to IDE project files for "
+      "convenience in editing even if they have not build rules."
+      ;
     }
   
   cmTypeMacro(cmAddCustomTargetCommand, cmCommand);
