@@ -78,6 +78,13 @@ void cmDocumentVariables::DefineVariables(cmake* cm)
      "useful for adding special flags and commands based"
      " on the final build environment. ", false, 
      "Variables that Provide Information");
+  cm->DefineProperty
+    ("CMAKE_CROSSCOMPILING", cmProperty::VARIABLE,
+     "Is CMake currently cross compiling.",
+     "This variable will be set to true by CMake if CMake is cross "
+     "compiling. Specifically if the build platform is different "
+     "from the target platform.", false,
+     "Variables that Provide Information");
   cm->DefineProperty
     ("CMAKE_CACHEFILE_DIR", cmProperty::VARIABLE,
      "The directory with the CMakeCache.txt file.",
@@ -201,6 +208,20 @@ void cmDocumentVariables::DefineVariables(cmake* cm)
      "This specifies the minor version of the CMake"
      " executable being run.",false,
      "Variables that Provide Information");
+  cm->DefineProperty
+    ("CMAKE_PATCH_VERSION", cmProperty::VARIABLE,
+     "The patch version of cmake (i.e. the 3 in X.X.3).",
+     "This specifies the patch version of the CMake"
+     " executable being run.",false,
+     "Variables that Provide Information");
+  cm->DefineProperty
+    ("CMAKE_VERSION", cmProperty::VARIABLE,
+     "The full version of cmake in major.minor.patch format.",
+     "This specifies the full version of the CMake executable being run.  "
+     "This variable is defined by versions 2.6.3 and higher.  "
+     "See variables CMAKE_MAJOR_VERSION, CMAKE_MINOR_VERSION, and "
+     "CMAKE_PATCH_VERSION for individual version components.", false,
+     "Variables that Provide Information");
 
   cm->DefineProperty
     ("CMAKE_PARENT_LIST_FILE", cmProperty::VARIABLE,
@@ -417,14 +438,6 @@ void cmDocumentVariables::DefineVariables(cmake* cm)
      "CMakeLists file.",false,
      "Variables That Change Behavior");
 
-  cm->DefineProperty
-    ("CMAKE_CROSSCOMPILING", cmProperty::VARIABLE,
-     "Is CMake currently cross compiling.",
-     "This variable will be set to true by CMake if CMake is cross "
-     "compiling. Specifically if the build platform is different "
-     "from the target platform.", false,
-     "Variables That Change Behavior");
-
   cm->DefineProperty
     ("CMAKE_INSTALL_PREFIX", cmProperty::VARIABLE,
      "Install directory used by install.",
@@ -443,6 +456,97 @@ void cmDocumentVariables::DefineVariables(cmake* cm)
      false,
      "Variables That Change Behavior");
 
+  cm->DefineProperty
+    ("CMAKE_PREFIX_PATH", cmProperty::VARIABLE,
+     "Path used for searching by FIND_XXX(), with appropriate suffixes added.",
+     "Specifies a path which will be used by the FIND_XXX() commands. It "
+     "contains the \"base\" directories, the FIND_XXX() commands append "
+     "appropriate subdirectories to the base directories. So FIND_PROGRAM() "
+     "adds /bin to each of the directories in the path, FIND_LIBRARY() "
+     "appends /lib to each of the directories, and FIND_PATH() and "
+     "FIND_FILE() append /include . By default it is empty, it is intended "
+     "to be set by the project. See also CMAKE_SYSTEM_PREFIX_PATH, "
+     "CMAKE_INCLUDE_PATH, CMAKE_LIBRARY_PATH, CMAKE_PROGRAM_PATH.", false,
+     "Variables That Change Behavior");
+
+  cm->DefineProperty
+    ("CMAKE_INCLUDE_PATH", cmProperty::VARIABLE,
+     "Path used for searching by FIND_FILE() and FIND_PATH().",
+     "Specifies a path which will be used both by FIND_FILE() and "
+     "FIND_PATH(). Both commands will check each of the contained directories "
+     "for the existence of the file which is currently searched. By default "
+     "it is empty, it is intended to be set by the project. See also "
+     "CMAKE_SYSTEM_INCLUDE_PATH, CMAKE_PREFIX_PATH.", false,
+     "Variables That Change Behavior");
+
+  cm->DefineProperty
+    ("CMAKE_LIBRARY_PATH", cmProperty::VARIABLE,
+     "Path used for searching by FIND_LIBRARY().",
+     "Specifies a path which will be used by FIND_LIBRARY(). FIND_LIBRARY() "
+     "will check each of the contained directories for the existence of the "
+     "library which is currently searched. By default it is empty, it is "
+     "intended to be set by the project. See also CMAKE_SYSTEM_LIBRARY_PATH, "
+     "CMAKE_PREFIX_PATH.", false,
+     "Variables That Change Behavior");
+
+  cm->DefineProperty
+    ("CMAKE_PROGRAM_PATH", cmProperty::VARIABLE,
+     "Path used for searching by FIND_PROGRAM().",
+     "Specifies a path which will be used by FIND_PROGRAM(). FIND_PROGRAM() "
+     "will check each of the contained directories for the existence of the "
+     "program which is currently searched. By default it is empty, it is "
+     "intended to be set by the project. See also CMAKE_SYSTEM_PROGRAM_PATH, "
+     " CMAKE_PREFIX_PATH.", false,
+     "Variables That Change Behavior");
+
+  cm->DefineProperty
+    ("CMAKE_SYSTEM_PREFIX_PATH", cmProperty::VARIABLE,
+     "Path used for searching by FIND_XXX(), with appropriate suffixes added.",
+     "Specifies a path which will be used by the FIND_XXX() commands. It "
+     "contains the \"base\" directories, the FIND_XXX() commands append "
+     "appropriate subdirectories to the base directories. So FIND_PROGRAM() "
+     "adds /bin to each of the directories in the path, FIND_LIBRARY() "
+     "appends /lib to each of the directories, and FIND_PATH() and "
+     "FIND_FILE() append /include . By default this contains the standard "
+     "directories for the current system. It is NOT intended "
+     "to be modified by the project, use CMAKE_PREFIX_PATH for this. See also "
+     "CMAKE_SYSTEM_INCLUDE_PATH, CMAKE_SYSTEM_LIBRARY_PATH, "
+     "CMAKE_SYSTEM_PROGRAM_PATH.", false,
+     "Variables That Change Behavior");
+
+  cm->DefineProperty
+    ("CMAKE_SYSTEM_INCLUDE_PATH", cmProperty::VARIABLE,
+     "Path used for searching by FIND_FILE() and FIND_PATH().",
+     "Specifies a path which will be used both by FIND_FILE() and "
+     "FIND_PATH(). Both commands will check each of the contained directories "
+     "for the existence of the file which is currently searched. By default "
+     "it contains the standard directories for the current system. It is "
+     "NOT intended to be modified by the project, use CMAKE_INCLUDE_PATH "
+     "for this. See also CMAKE_SYSTEM_PREFIX_PATH.", false,
+     "Variables That Change Behavior");
+
+  cm->DefineProperty
+    ("CMAKE_SYSTEM_LIBRARY_PATH", cmProperty::VARIABLE,
+     "Path used for searching by FIND_LIBRARY().",
+     "Specifies a path which will be used by FIND_LIBRARY(). FIND_LIBRARY() "
+     "will check each of the contained directories for the existence of the "
+     "library which is currently searched. By default it contains the "
+     "standard directories for the current system. It is NOT intended to be "
+     "modified by the project, use CMAKE_SYSTEM_LIBRARY_PATH for this. See "
+     "also CMAKE_SYSTEM_PREFIX_PATH.", false,
+     "Variables That Change Behavior");
+
+  cm->DefineProperty
+    ("CMAKE_SYSTEM_PROGRAM_PATH", cmProperty::VARIABLE,
+     "Path used for searching by FIND_PROGRAM().",
+     "Specifies a path which will be used by FIND_PROGRAM(). FIND_PROGRAM() "
+     "will check each of the contained directories for the existence of the "
+     "program which is currently searched. By default it contains the "
+     "standard directories for the current system. It is NOT intended to be "
+     "modified by the project, use CMAKE_PROGRAM_PATH for this. See also "
+     "CMAKE_SYSTEM_PREFIX_PATH.", false,
+     "Variables That Change Behavior");
+
   cm->DefineProperty
     ("CMAKE_USER_MAKE_RULES_OVERRIDE", cmProperty::VARIABLE,
      "Specify a file that can change the build rule variables.",
@@ -486,9 +590,16 @@ void cmDocumentVariables::DefineVariables(cmake* cm)
      "add_executable(CMakeSetup WIN32 ${SRCS})\n",false,
      "Variables That Change Behavior");
 
-  
+  cm->DefineProperty
+    ("CMAKE_COLOR_MAKEFILE", cmProperty::VARIABLE,
+     "Enables color output when using the Makefile generator.",
+     "When enabled, the generated Makefiles will produce colored output. "
+     "Default is ON.",false,
+     "Variables That Change Behavior");
+
+
   // Variables defined by CMake that describe the system
-  
+
   cm->DefineProperty
     ("CMAKE_SYSTEM", cmProperty::VARIABLE,
      "Name of system cmake is compiling for.",
@@ -646,6 +757,24 @@ void cmDocumentVariables::DefineVariables(cmake* cm)
      "Set to true when the host system is Windows and on cygwin.",false,
      "Variables That Describe the System");
 
+  cm->DefineProperty
+    ("CMAKE_OBJECT_PATH_MAX", cmProperty::VARIABLE,
+     "Maximum object file full-path length allowed by native build tools.",
+     "CMake computes for every source file an object file name that is "
+     "unique to the source file and deterministic with respect to the "
+     "full path to the source file.  "
+     "This allows multiple source files in a target to share the same name "
+     "if they lie in different directories without rebuilding when one is "
+     "added or removed.  "
+     "However, it can produce long full paths in a few cases, so CMake "
+     "shortens the path using a hashing scheme when the full path to an "
+     "object file exceeds a limit.  "
+     "CMake has a built-in limit for each platform that is sufficient for "
+     "common tools, but some native tools may have a lower limit.  "
+     "This variable may be set to specify the limit explicitly.  "
+     "The value must be an integer no less than 128.",false,
+     "Variables That Describe the System");
+
   // Variables that affect the building of object files and 
   // targets.
   //
@@ -714,18 +843,23 @@ void cmDocumentVariables::DefineVariables(cmake* cm)
      "See that target property for additional information.",
      false,
      "Variables that Control the Build");
-  
 
   cm->DefineProperty
     ("CMAKE_DEBUG_POSTFIX", cmProperty::VARIABLE,
-     "A postfix to add to targets when build as debug.",
-     "This variable is used to initialize the DEBUG_POSTFIX "
-     "property on all the targets. If set the postfix will be "
-     "appended to any targets built when the configuration is "
-     "Debug.",
+     "See variable CMAKE_<CONFIG>_POSTFIX.",
+     "This variable is a special case of the more-general "
+     "CMAKE_<CONFIG>_POSTFIX variable for the DEBUG configuration.",
      false,
      "Variables that Control the Build");
-  
+  cm->DefineProperty
+    ("CMAKE_<CONFIG>_POSTFIX", cmProperty::VARIABLE,
+     "Default filename postfix for libraries under configuration <CONFIG>.",
+     "When a non-executable target is created its <CONFIG>_POSTFIX "
+     "target property is initialized with the value of this variable "
+     "if it is set.",
+     false,
+     "Variables that Control the Build");
+
   cm->DefineProperty
     ("CMAKE_BUILD_WITH_INSTALL_RPATH", cmProperty::VARIABLE,
      "Use the install path for the RPATH",
