@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmGlobalGenerator.h,v $
   Language:  C++
-  Date:      $Date: 2008-02-14 21:42:29 $
-  Version:   $Revision: 1.107 $
+  Date:      $Date: 2008-10-24 15:18:46 $
+  Version:   $Revision: 1.107.2.6 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -139,6 +139,9 @@ public:
 
   void AddInstallComponent(const char* component);
 
+  const std::set<cmStdString>* GetInstallComponents() const 
+  { return &InstallComponents; }
+
   ///! Add one installed target to the sets of the exports
   void AddTargetToExports(const char* exportSet, cmTarget* target, 
                           cmInstallTargetGenerator* archive,
@@ -210,12 +213,12 @@ public:
       configuration.  This is valid during generation only.  */
   cmTargetManifest const& GetTargetManifest() { return this->TargetManifest; }
 
-  /** Get the content of a directory on disk including the target
-      files to be generated.  This may be called only during the
-      generation step.  It is intended for use only by
-      cmComputeLinkInformation.  */
+  /** Get the content of a directory.  Directory listings are loaded
+      from disk at most once and cached.  During the generation step
+      the content will include the target files to be built even if
+      they do not yet exist.  */
   std::set<cmStdString> const& GetDirectoryContent(std::string const& dir,
-                                                   bool needDisk);
+                                                   bool needDisk = true);
 
   void AddTarget(cmTargets::value_type &v);
 
@@ -245,6 +248,10 @@ public:
   void FileReplacedDuringGenerate(const std::string& filename);
   void GetFilesReplacedDuringGenerate(std::vector<std::string>& filenames);
 
+  void AddRuleHash(const std::vector<std::string>& outputs,
+                   std::vector<std::string>::const_iterator first,
+                   std::vector<std::string>::const_iterator last);
+
 protected:
   // for a project collect all its targets by following depend
   // information, and also collect all the targets
@@ -257,9 +264,12 @@ protected:
                         projectTargets);
   void SetLanguageEnabledFlag(const char* l, cmMakefile* mf);
   void SetLanguageEnabledMaps(const char* l, cmMakefile* mf);
+  void FillExtensionToLanguageMap(const char* l, cmMakefile* mf);
 
   virtual bool CheckALLOW_DUPLICATE_CUSTOM_TARGETS();
 
+  bool CheckTargets();
+
   // Fill the ProjectMap, this must be called after LocalGenerators 
   // has been populated.
   void FillProjectMap();
@@ -270,7 +280,8 @@ protected:
   void CreateDefaultGlobalTargets(cmTargets* targets);
   cmTarget CreateGlobalTarget(const char* name, const char* message,
     const cmCustomCommandLines* commandLines,
-    std::vector<std::string> depends, bool depends_on_all = false);
+    std::vector<std::string> depends, const char* workingDir,
+                              bool depends_on_all = false);
 
   bool NeedSymbolicMark;
   bool UseLinkScript;
@@ -291,6 +302,7 @@ protected:
   bool InstallTargetEnabled;
   // Sets of named target exports
   std::map<cmStdString, std::vector<cmTargetExport*> > ExportSets;
+  void ClearExportSets();
 
   // Manifest of all targets that will be built for each configuration.
   // This is computed just before local generators generate.
@@ -310,6 +322,11 @@ private:
   // this is used to improve performance
   std::map<cmStdString,cmTarget *> TotalTargets;
 
+  // Record hashes for rules and outputs.
+  struct RuleHash { char Data[32]; };
+  std::map<cmStdString, RuleHash> RuleHashes;
+  void CheckRuleHashes();
+
   cmExternalMakefileProjectGenerator* ExtraGenerator;
 
   // track files replaced during a Generate
