@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmInstallGenerator.cxx,v $
   Language:  C++
-  Date:      $Date: 2008-01-28 13:38:35 $
-  Version:   $Revision: 1.15 $
+  Date:      $Date: 2009-01-13 18:03:52 $
+  Version:   $Revision: 1.15.2.1 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -189,9 +189,9 @@ std::string
 cmInstallGenerator::CreateComponentTest(const char* component)
 {
   std::string result = "NOT CMAKE_INSTALL_COMPONENT OR "
-    "\"${CMAKE_INSTALL_COMPONENT}\" MATCHES \"^(";
+    "\"${CMAKE_INSTALL_COMPONENT}\" STREQUAL \"";
   result += component;
-  result += ")$\"";
+  result += "\"";
   return result;
 }
 
