@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmLocalGenerator.cxx,v $
   Language:  C++
-  Date:      $Date: 2008-05-01 16:35:39 $
-  Version:   $Revision: 1.269.2.2 $
+  Date:      $Date: 2009-02-04 22:04:49 $
+  Version:   $Revision: 1.269.2.10 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -101,6 +101,44 @@ void cmLocalGenerator::Configure()
   // relative paths.
   this->UseRelativePaths = this->Makefile->IsOn("CMAKE_USE_RELATIVE_PATHS");
 
+  // Choose a maximum object file name length.
+  {
+#if defined(_WIN32) || defined(__CYGWIN__)
+  this->ObjectPathMax = 250;
+#else
+  this->ObjectPathMax = 1000;
+#endif
+  const char* plen = this->Makefile->GetDefinition("CMAKE_OBJECT_PATH_MAX");
+  if(plen && *plen)
+    {
+    unsigned int pmax;
+    if(sscanf(plen, "%u", &pmax) == 1)
+      {
+      if(pmax >= 128)
+        {
+        this->ObjectPathMax = pmax;
+        }
+      else
+        {
+        cmOStringStream w;
+        w << "CMAKE_OBJECT_PATH_MAX is set to " << pmax
+          << ", which is less than the minimum of 128.  "
+          << "The value will be ignored.";
+        this->Makefile->IssueMessage(cmake::AUTHOR_WARNING, w.str());
+        }
+      }
+    else
+      {
+      cmOStringStream w;
+      w << "CMAKE_OBJECT_PATH_MAX is set to \"" << plen
+        << "\", which fails to parse as a positive integer.  "
+        << "The value will be ignored.";
+      this->Makefile->IssueMessage(cmake::AUTHOR_WARNING, w.str());
+      }
+    }
+  this->ObjectMaxPathViolations.clear();
+  }
+
   this->Configured = true;
 
   this->GetGlobalGenerator()->SetCurrentLocalGenerator(previousLg);
@@ -608,10 +646,10 @@ void cmLocalGenerator::AddBuildTargetRule(const char* llang, cmTarget& target)
        !sf->GetPropertyAsBool("HEADER_FILE_ONLY") &&
        !sf->GetPropertyAsBool("EXTERNAL_OBJECT"))
       {
-      std::string::size_type dir_len = 0;
-      dir_len += strlen(this->Makefile->GetCurrentOutputDirectory());
-      dir_len += 1;
-      std::string obj = this->GetObjectFileNameWithoutTarget(*sf, dir_len);
+      std::string dir_max;
+      dir_max += this->Makefile->GetCurrentOutputDirectory();
+      dir_max += "/";
+      std::string obj = this->GetObjectFileNameWithoutTarget(*sf, dir_max);
       if(!obj.empty())
         {
         std::string ofname = this->Makefile->GetCurrentOutputDirectory();
@@ -723,11 +761,7 @@ void cmLocalGenerator
           }
         }
         break; 
-      case cmTarget::UTILITY:
-      case cmTarget::GLOBAL_TARGET:
-      case cmTarget::INSTALL_FILES:
-      case cmTarget::INSTALL_PROGRAMS:
-      case cmTarget::INSTALL_DIRECTORY:
+      default:
         break;
       }
     }
@@ -1041,22 +1075,54 @@ cmLocalGenerator::ExpandRuleVariables(std::string& s,
   s = expandedInput;
 }
 
-
-std::string 
-cmLocalGenerator::ConvertToOutputForExisting(const char* p)
+//----------------------------------------------------------------------------
+std::string
+cmLocalGenerator::ConvertToOutputForExistingCommon(const char* remote,
+                                                   std::string const& result)
 {
-  std::string ret = p;
-  if(this->WindowsShell && ret.find(' ') != ret.npos 
-     && cmSystemTools::FileExists(p))
+  // If this is a windows shell, the result has a space, and the path
+  // already exists, we can use a short-path to reference it without a
+  // space.
+  if(this->WindowsShell && result.find(' ') != result.npos &&
+     cmSystemTools::FileExists(remote))
     {
-    if(cmSystemTools::GetShortPath(p, ret))
+    std::string tmp;
+    if(cmSystemTools::GetShortPath(remote, tmp))
       {
-      return  this->Convert(ret.c_str(), NONE, SHELL, true);
+      return this->Convert(tmp.c_str(), NONE, SHELL, true);
       }
     }
-  return this->Convert(p, START_OUTPUT, SHELL, true);
+
+  // Otherwise, leave it unchanged.
+  return result;
 }
 
+//----------------------------------------------------------------------------
+std::string
+cmLocalGenerator::ConvertToOutputForExisting(const char* remote,
+                                             RelativeRoot local)
+{
+  // Perform standard conversion.
+  std::string result = this->Convert(remote, local, SHELL, true);
+
+  // Consider short-path.
+  return this->ConvertToOutputForExistingCommon(remote, result);
+}
+
+//----------------------------------------------------------------------------
+std::string
+cmLocalGenerator::ConvertToOutputForExisting(RelativeRoot remote,
+                                             const char* local)
+{
+  // Perform standard conversion.
+  std::string result = this->Convert(remote, local, SHELL, true);
+
+  // Consider short-path.
+  const char* remotePath = this->GetRelativeRootPath(remote);
+  return this->ConvertToOutputForExistingCommon(remotePath, result);
+}
+
+//----------------------------------------------------------------------------
 const char* cmLocalGenerator::GetIncludeFlags(const char* lang)
 {
   if(!lang)
@@ -1122,9 +1188,11 @@ const char* cmLocalGenerator::GetIncludeFlags(const char* lang)
       frameworkDir = cmSystemTools::CollapseFullPath(frameworkDir.c_str());
       if(emitted.insert(frameworkDir).second)
         {
-        includeFlags 
-          << "-F" 
-          << this->ConvertToOutputForExisting(frameworkDir.c_str()) << " ";
+        includeFlags
+          << "-F" << this->Convert(frameworkDir.c_str(),
+                                   cmLocalGenerator::START_OUTPUT,
+                                   cmLocalGenerator::SHELL, true)
+          << " ";
         }
       continue;
       }
@@ -1441,11 +1509,7 @@ void cmLocalGenerator::GetTargetFlags(std::string& linkLibs,
         }
       }
       break; 
-    case cmTarget::UTILITY:
-    case cmTarget::GLOBAL_TARGET:
-    case cmTarget::INSTALL_FILES:
-    case cmTarget::INSTALL_PROGRAMS:
-    case cmTarget::INSTALL_DIRECTORY:
+    default:
       break;
     }
 }
@@ -1573,7 +1637,7 @@ void cmLocalGenerator::OutputLinkLibraries(std::ostream& fout,
         ri != runtimeDirs.end(); ++ri)
       {
       rpath += cli.GetRuntimeFlag();
-      rpath += this->Convert(ri->c_str(), FULL, SHELL, false);
+      rpath += this->Convert(ri->c_str(), NONE, SHELL, false);
       rpath += " ";
       }
     fout << rpath;
@@ -1630,7 +1694,7 @@ void cmLocalGenerator::AddLanguageFlags(std::string& flags,
     const char* sysrootDefault = 
       this->Makefile->GetDefinition("CMAKE_OSX_SYSROOT_DEFAULT");
     bool flagsUsed = false;
-    if(osxArch && sysroot  && lang && lang[0] =='C')
+    if(osxArch && sysroot  && lang && (lang[0] =='C' || lang[0] == 'F'))
       { 
       std::vector<std::string> archs;
       cmSystemTools::ExpandListArgument(std::string(osxArch),
@@ -1714,6 +1778,7 @@ std::string cmLocalGenerator::GetRealDependency(const char* inName,
       case cmTarget::STATIC_LIBRARY:
       case cmTarget::SHARED_LIBRARY:
       case cmTarget::MODULE_LIBRARY:
+      case cmTarget::UNKNOWN_LIBRARY:
         {
         // Get the location of the target's output file and depend on it.
         if(const char* location = target->GetLocation(config))
@@ -1740,6 +1805,15 @@ std::string cmLocalGenerator::GetRealDependency(const char* inName,
     // This is a full path.  Return it as given.
     return inName;
     }
+
+  // Check for a source file in this directory that matches the
+  // dependency.
+  if(cmSourceFile* sf = this->Makefile->GetSource(inName))
+    {
+    name = sf->GetFullPath();
+    return name;
+    }
+
   // Treat the name as relative to the source directory in which it
   // was given.
   name = this->Makefile->GetCurrentDirectory();
@@ -1944,7 +2018,21 @@ cmLocalGenerator::ConvertToOptionallyRelativeOutputPath(const char* remote)
 }
 
 //----------------------------------------------------------------------------
-std::string cmLocalGenerator::Convert(const char* source, 
+const char* cmLocalGenerator::GetRelativeRootPath(RelativeRoot relroot)
+{
+  switch (relroot)
+    {
+    case HOME:         return this->Makefile->GetHomeDirectory();
+    case START:        return this->Makefile->GetStartDirectory();
+    case HOME_OUTPUT:  return this->Makefile->GetHomeOutputDirectory();
+    case START_OUTPUT: return this->Makefile->GetStartOutputDirectory();
+    default: break;
+    }
+  return 0;
+}
+
+//----------------------------------------------------------------------------
+std::string cmLocalGenerator::Convert(const char* source,
                                       RelativeRoot relative,
                                       OutputFormat output,
                                       bool optional)
@@ -1992,7 +2080,15 @@ std::string cmLocalGenerator::Convert(const char* source,
         break;
       }
     }
-  // Now convert it to an output path.
+  return this->ConvertToOutputFormat(result.c_str(), output);
+}
+
+//----------------------------------------------------------------------------
+std::string cmLocalGenerator::ConvertToOutputFormat(const char* source,
+                                                    OutputFormat output)
+{
+  std::string result = source;
+  // Convert it to an output path.
   if (output == MAKEFILE)
     {
     result = cmSystemTools::ConvertToOutputPath(result.c_str());
@@ -2024,6 +2120,40 @@ std::string cmLocalGenerator::Convert(const char* source,
   return result;
 }
 
+//----------------------------------------------------------------------------
+std::string cmLocalGenerator::Convert(RelativeRoot remote,
+                                      const char* local,
+                                      OutputFormat output,
+                                      bool optional)
+{
+  const char* remotePath = this->GetRelativeRootPath(remote);
+  if(local && (!optional || this->UseRelativePaths))
+    {
+    std::vector<std::string> components;
+    std::string result;
+    switch(remote)
+      {
+      case HOME:
+      case HOME_OUTPUT:
+      case START:
+      case START_OUTPUT:
+        cmSystemTools::SplitPath(local, components);
+        result = this->ConvertToRelativePath(components, remotePath);
+        break;
+      case FULL:
+        result = remotePath;
+        break;
+      case NONE:
+        break;
+      }
+    return this->ConvertToOutputFormat(result.c_str(), output);
+    }
+  else
+    {
+    return this->ConvertToOutputFormat(remotePath, output);
+    }
+}
+
 //----------------------------------------------------------------------------
 std::string cmLocalGenerator::FindRelativePathTopSource()
 {
@@ -2230,6 +2360,10 @@ cmLocalGenerator
       // We also skip over the leading slash given by the user.
       std::string destination = l->second.GetInstallPath().substr(1);
       cmSystemTools::ConvertToUnixSlashes(destination);
+      if(destination.empty())
+        {
+        destination = ".";
+        }
 
       // Generate the proper install generator for this target type.
       switch(l->second.GetType())
@@ -2312,16 +2446,11 @@ cmLocalGeneratorShortenObjectName(std::string& objName,
     }
 }
 
-static bool cmLocalGeneratorCheckObjectName(std::string& objName,
-                                            std::string::size_type dir_len)
+static
+bool cmLocalGeneratorCheckObjectName(std::string& objName,
+                                     std::string::size_type dir_len,
+                                     std::string::size_type max_total_len)
 {
-  // Choose a maximum file name length.
-#if defined(_WIN32) || defined(__CYGWIN__)
-  std::string::size_type const max_total_len = 250;
-#else
-  std::string::size_type const max_total_len = 1000;
-#endif
-
   // Enforce the maximum file name length if possible.
   std::string::size_type max_obj_len = max_total_len;
   if(dir_len < max_total_len)
@@ -2351,7 +2480,7 @@ static bool cmLocalGeneratorCheckObjectName(std::string& objName,
 std::string&
 cmLocalGenerator
 ::CreateSafeUniqueObjectFileName(const char* sin,
-                                 std::string::size_type dir_len)
+                                 std::string const& dir_max)
 {
   // Look for an existing mapped name for this object file.
   std::map<cmStdString,cmStdString>::iterator it =
@@ -2365,7 +2494,9 @@ cmLocalGenerator
 
     // Avoid full paths by removing leading slashes.
     std::string::size_type pos = 0;
-    for(;pos < ssin.size() && ssin[pos] == '/'; ++pos);
+    for(;pos < ssin.size() && ssin[pos] == '/'; ++pos)
+      {
+      }
     ssin = ssin.substr(pos);
 
     // Avoid full paths by removing colons.
@@ -2410,9 +2541,28 @@ cmLocalGenerator
       }
 
 #if defined(CM_LG_ENCODE_OBJECT_NAMES)
-    cmLocalGeneratorCheckObjectName(ssin, dir_len);
+    if(!cmLocalGeneratorCheckObjectName(ssin, dir_max.size(),
+                                        this->ObjectPathMax))
+      {
+      // Warn if this is the first time the path has been seen.
+      if(this->ObjectMaxPathViolations.insert(dir_max).second)
+        {
+        cmOStringStream m;
+        m << "The object file directory\n"
+          << "  " << dir_max << "\n"
+          << "has " << dir_max.size() << " characters.  "
+          << "The maximum full path to an object file is "
+          << this->ObjectPathMax << " characters "
+          << "(see CMAKE_OBJECT_PATH_MAX).  "
+          << "Object file\n"
+          << "  " << ssin << "\n"
+          << "cannot be safely placed under this directory.  "
+          << "The build may not work correctly.";
+        this->Makefile->IssueMessage(cmake::WARNING, m.str());
+        }
+      }
 #else
-    (void)dir_len;
+    (void)dir_max;
 #endif
 
     // Insert the newly mapped object file name.
@@ -2428,7 +2578,7 @@ cmLocalGenerator
 std::string
 cmLocalGenerator
 ::GetObjectFileNameWithoutTarget(const cmSourceFile& source,
-                                 std::string::size_type dir_len,
+                                 std::string const& dir_max,
                                  bool* hasSourceExtension)
 {
   // Construct the object file name using the full path to the source
@@ -2516,7 +2666,7 @@ cmLocalGenerator
     }
 
   // Convert to a safe name.
-  return this->CreateSafeUniqueObjectFileName(objectName.c_str(), dir_len);
+  return this->CreateSafeUniqueObjectFileName(objectName.c_str(), dir_max);
 }
 
 //----------------------------------------------------------------------------
@@ -2577,7 +2727,7 @@ std::string cmLocalGenerator::EscapeForShell(const char* str, bool makeVars,
     {
     flags |= cmsysSystem_Shell_Flag_VSIDE;
     }
-  else
+  else if(!this->LinkScriptShell)
     {
     flags |= cmsysSystem_Shell_Flag_Make;
     }
@@ -2776,16 +2926,11 @@ bool cmLocalGenerator::CheckDefinition(std::string const& define) const
 }
 
 //----------------------------------------------------------------------------
-static std::string cmLGInfoProp(cmTarget* target, const char* prop)
+static void cmLGInfoProp(cmMakefile* mf, cmTarget* target, const char* prop)
 {
   if(const char* val = target->GetProperty(prop))
     {
-    return val;
-    }
-  else
-    {
-    // For compatibility check for a variable.
-    return target->GetMakefile()->GetSafeDefinition(prop);
+    mf->AddDefinition(prop, val);
     }
 }
 
@@ -2794,66 +2939,81 @@ void cmLocalGenerator::GenerateAppleInfoPList(cmTarget* target,
                                               const char* targetName,
                                               const char* fname)
 {
-  std::string info_EXECUTABLE_NAME = targetName;
+  // Find the Info.plist template.
+  const char* in = target->GetProperty("MACOSX_BUNDLE_INFO_PLIST");
+  std::string inFile = (in && *in)? in : "MacOSXBundleInfo.plist.in";
+  if(!cmSystemTools::FileIsFullPath(inFile.c_str()))
+    {
+    std::string inMod = this->Makefile->GetModulesFile(inFile.c_str());
+    if(!inMod.empty())
+      {
+      inFile = inMod;
+      }
+    }
+  if(!cmSystemTools::FileExists(inFile.c_str(), true))
+    {
+    cmOStringStream e;
+    e << "Target " << target->GetName() << " Info.plist template \""
+      << inFile << "\" could not be found.";
+    cmSystemTools::Error(e.str().c_str());
+    return;
+    }
 
-  // Lookup the properties.
-  std::string info_INFO_STRING =
-    cmLGInfoProp(target, "MACOSX_BUNDLE_INFO_STRING");
-  std::string info_ICON_FILE =
-    cmLGInfoProp(target, "MACOSX_BUNDLE_ICON_FILE");
-  std::string info_GUI_IDENTIFIER =
-    cmLGInfoProp(target, "MACOSX_BUNDLE_GUI_IDENTIFIER");
-  std::string info_LONG_VERSION_STRING =
-    cmLGInfoProp(target, "MACOSX_BUNDLE_LONG_VERSION_STRING");
-  std::string info_BUNDLE_NAME =
-    cmLGInfoProp(target, "MACOSX_BUNDLE_BUNDLE_NAME");
-  std::string info_SHORT_VERSION_STRING =
-    cmLGInfoProp(target, "MACOSX_BUNDLE_SHORT_VERSION_STRING");
-  std::string info_BUNDLE_VERSION =
-    cmLGInfoProp(target, "MACOSX_BUNDLE_BUNDLE_VERSION");
-  std::string info_COPYRIGHT =
-    cmLGInfoProp(target, "MACOSX_BUNDLE_COPYRIGHT");
+  // Convert target properties to variables in an isolated makefile
+  // scope to configure the file.  If properties are set they will
+  // override user make variables.  If not the configuration will fall
+  // back to the directory-level values set by the user.
+  cmMakefile* mf = this->Makefile;
+  mf->PushScope();
+  mf->AddDefinition("MACOSX_BUNDLE_EXECUTABLE_NAME", targetName);
+  cmLGInfoProp(mf, target, "MACOSX_BUNDLE_INFO_STRING");
+  cmLGInfoProp(mf, target, "MACOSX_BUNDLE_ICON_FILE");
+  cmLGInfoProp(mf, target, "MACOSX_BUNDLE_GUI_IDENTIFIER");
+  cmLGInfoProp(mf, target, "MACOSX_BUNDLE_LONG_VERSION_STRING");
+  cmLGInfoProp(mf, target, "MACOSX_BUNDLE_BUNDLE_NAME");
+  cmLGInfoProp(mf, target, "MACOSX_BUNDLE_SHORT_VERSION_STRING");
+  cmLGInfoProp(mf, target, "MACOSX_BUNDLE_BUNDLE_VERSION");
+  cmLGInfoProp(mf, target, "MACOSX_BUNDLE_COPYRIGHT");
+  mf->ConfigureFile(inFile.c_str(), fname, false, false, false);
+  mf->PopScope();
+}
 
-  // Generate the file.
-  cmGeneratedFileStream fout(fname);
-  fout.SetCopyIfDifferent(true);
-  fout <<
-    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
-    "<!DOCTYPE plist PUBLIC \"-//Apple Computer//DTD PLIST 1.0//EN\"\n"
-    "  \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n"
-    "<plist version=\"1.0\">\n"
-    "<dict>\n"
-    "\t<key>CFBundleDevelopmentRegion</key>\n"
-    "\t<string>English</string>\n"
-    "\t<key>CFBundleExecutable</key>\n"
-    "\t<string>" << info_EXECUTABLE_NAME << "</string>\n"
-    "\t<key>CFBundleGetInfoString</key>\n"
-    "\t<string>" << info_INFO_STRING << "</string>\n"
-    "\t<key>CFBundleIconFile</key>\n"
-    "\t<string>" << info_ICON_FILE << "</string>\n"
-    "\t<key>CFBundleIdentifier</key>\n"
-    "\t<string>" << info_GUI_IDENTIFIER << "</string>\n"
-    "\t<key>CFBundleInfoDictionaryVersion</key>\n"
-    "\t<string>6.0</string>\n"
-    "\t<key>CFBundleLongVersionString</key>\n"
-    "\t<string>" << info_LONG_VERSION_STRING << "</string>\n"
-    "\t<key>CFBundleName</key>\n"
-    "\t<string>" << info_BUNDLE_NAME << "</string>\n"
-    "\t<key>CFBundlePackageType</key>\n"
-    "\t<string>APPL</string>\n"
-    "\t<key>CFBundleShortVersionString</key>\n"
-    "\t<string>" << info_SHORT_VERSION_STRING << "</string>\n"
-    "\t<key>CFBundleSignature</key>\n"
-    "\t<string>????" /* break string to avoid trigraph */ "</string>\n"
-    "\t<key>CFBundleVersion</key>\n"
-    "\t<string>" << info_BUNDLE_VERSION << "</string>\n"
-    "\t<key>CSResourcesFileMapped</key>\n"
-    "\t<true/>\n"
-    "\t<key>LSRequiresCarbon</key>\n"
-    "\t<true/>\n"
-    "\t<key>NSHumanReadableCopyright</key>\n"
-    "\t<string>" << info_COPYRIGHT << "</string>\n"
-    "</dict>\n"
-    "</plist>\n"
-    ;
+//----------------------------------------------------------------------------
+void cmLocalGenerator::GenerateFrameworkInfoPList(cmTarget* target,
+                                                  const char* targetName,
+                                                  const char* fname)
+{
+  // Find the Info.plist template.
+  const char* in = target->GetProperty("MACOSX_FRAMEWORK_INFO_PLIST");
+  std::string inFile = (in && *in)? in : "MacOSXFrameworkInfo.plist.in";
+  if(!cmSystemTools::FileIsFullPath(inFile.c_str()))
+    {
+    std::string inMod = this->Makefile->GetModulesFile(inFile.c_str());
+    if(!inMod.empty())
+      {
+      inFile = inMod;
+      }
+    }
+  if(!cmSystemTools::FileExists(inFile.c_str(), true))
+    {
+    cmOStringStream e;
+    e << "Target " << target->GetName() << " Info.plist template \""
+      << inFile << "\" could not be found.";
+    cmSystemTools::Error(e.str().c_str());
+    return;
+    }
+
+  // Convert target properties to variables in an isolated makefile
+  // scope to configure the file.  If properties are set they will
+  // override user make variables.  If not the configuration will fall
+  // back to the directory-level values set by the user.
+  cmMakefile* mf = this->Makefile;
+  mf->PushScope();
+  mf->AddDefinition("MACOSX_FRAMEWORK_NAME", targetName);
+  cmLGInfoProp(mf, target, "MACOSX_FRAMEWORK_ICON_FILE");
+  cmLGInfoProp(mf, target, "MACOSX_FRAMEWORK_IDENTIFIER");
+  cmLGInfoProp(mf, target, "MACOSX_FRAMEWORK_SHORT_VERSION_STRING");
+  cmLGInfoProp(mf, target, "MACOSX_FRAMEWORK_BUNDLE_VERSION");
+  mf->ConfigureFile(inFile.c_str(), fname, false, false, false);
+  mf->PopScope();
 }
