@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmLocalGenerator.h,v $
   Language:  C++
-  Date:      $Date: 2008-02-14 20:31:08 $
-  Version:   $Revision: 1.103 $
+  Date:      $Date: 2009-01-13 18:03:52 $
+  Version:   $Revision: 1.103.2.3 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -108,10 +108,18 @@ public:
    */
   enum RelativeRoot { NONE, FULL, HOME, START, HOME_OUTPUT, START_OUTPUT };
   enum OutputFormat { UNCHANGED, MAKEFILE, SHELL };
-  std::string Convert(const char* source, 
-                      RelativeRoot relative, 
+  std::string ConvertToOutputFormat(const char* source, OutputFormat output);
+  std::string Convert(const char* remote, RelativeRoot local,
                       OutputFormat output = UNCHANGED,
                       bool optional = false);
+  std::string Convert(RelativeRoot remote, const char* local,
+                      OutputFormat output = UNCHANGED,
+                      bool optional = false);
+
+  /**
+    * Get path for the specified relative root.
+    */
+  const char* GetRelativeRootPath(RelativeRoot relroot);
   
   /**
    * Convert the given path to an output path that is optionally
@@ -162,7 +170,13 @@ public:
   std::string GetRealLocation(const char* inName, const char* config);
 
   ///! for existing files convert to output path and short path if spaces
-  std::string ConvertToOutputForExisting(const char* p);
+  std::string ConvertToOutputForExisting(const char* remote,
+                                         RelativeRoot local = START_OUTPUT);
+
+  /** For existing path identified by RelativeRoot convert to output
+      path and short path if spaces.  */
+  std::string ConvertToOutputForExisting(RelativeRoot remote,
+                                         const char* local = 0);
   
   /** Called from command-line hook to clear dependencies.  */
   virtual void ClearDependencies(cmMakefile* /* mf */, 
@@ -275,6 +289,13 @@ public:
    */
   void GenerateAppleInfoPList(cmTarget* target, const char* targetName,
                               const char* fname);
+
+  /**
+   * Generate a Mac OS X framework Info.plist file.
+   */
+  void GenerateFrameworkInfoPList(cmTarget* target,
+                                  const char* targetName,
+                                  const char* fname);
 protected:
   /** Construct a comment for a custom command.  */
   std::string ConstructComment(const cmCustomCommand& cc,
@@ -319,10 +340,10 @@ protected:
 
   // Compute object file names.
   std::string GetObjectFileNameWithoutTarget(const cmSourceFile& source,
-                                             std::string::size_type dir_len,
+                                             std::string const& dir_max,
                                              bool* hasSourceExtension = 0);
   std::string& CreateSafeUniqueObjectFileName(const char* sin,
-                                              std::string::size_type dir_len);
+                                              std::string const& dir_max);
 
   void ConfigureRelativePaths();
   std::string FindRelativePathTopSource();
@@ -348,6 +369,8 @@ protected:
   std::vector<cmLocalGenerator*> Children;
   std::map<cmStdString, cmStdString> LanguageToIncludeFlags;
   std::map<cmStdString, cmStdString> UniqueObjectNamesMap;
+  std::string::size_type ObjectPathMax;
+  std::set<cmStdString> ObjectMaxPathViolations;
   bool WindowsShell;
   bool WindowsVSIDE;
   bool WatcomWMake;
@@ -378,6 +401,9 @@ protected:
 
   unsigned int BackwardsCompatibility;
   bool BackwardsCompatibilityFinal;
+private:
+  std::string ConvertToOutputForExistingCommon(const char* remote,
+                                               std::string const& result);
 };
 
 #endif
