@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmLocalUnixMakefileGenerator3.cxx,v $
   Language:  C++
-  Date:      $Date: 2008-05-01 16:35:39 $
-  Version:   $Revision: 1.240.2.5 $
+  Date:      $Date: 2009-01-13 18:03:52 $
+  Version:   $Revision: 1.240.2.8 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -358,10 +358,10 @@ cmLocalUnixMakefileGenerator3
     commands.push_back(
       this->GetRecursiveMakeCall(tgtMakefileName.c_str(), targetName.c_str())
       );
-    this->CreateCDCommand(commands,
-                          this->Makefile->GetHomeOutputDirectory(),
-                          this->Makefile->GetStartOutputDirectory());
     }
+  this->CreateCDCommand(commands,
+                        this->Makefile->GetHomeOutputDirectory(),
+                        cmLocalGenerator::START_OUTPUT);
 
   // Write the rule to the makefile.
   std::vector<std::string> no_depends;
@@ -404,7 +404,7 @@ void cmLocalUnixMakefileGenerator3
                          (makefile2.c_str(),localName.c_str()));
       this->CreateCDCommand(commands,
                             this->Makefile->GetHomeOutputDirectory(),
-                            this->Makefile->GetStartOutputDirectory());
+                            cmLocalGenerator::START_OUTPUT);
       this->WriteMakeRule(ruleFileStream, "Convenience name for target.",
                           localName.c_str(), depends, commands, true);
       
@@ -432,7 +432,7 @@ void cmLocalUnixMakefileGenerator3
                          (makefileName.c_str(), makeTargetName.c_str()));
       this->CreateCDCommand(commands,
                             this->Makefile->GetHomeOutputDirectory(),
-                            this->Makefile->GetStartOutputDirectory());
+                            cmLocalGenerator::START_OUTPUT);
       this->WriteMakeRule(ruleFileStream, "fast build rule for target.",
                           localName.c_str(), depends, commands, true);
 
@@ -450,7 +450,7 @@ void cmLocalUnixMakefileGenerator3
                            (makefile2.c_str(), makeTargetName.c_str()));
         this->CreateCDCommand(commands,
                               this->Makefile->GetHomeOutputDirectory(),
-                              this->Makefile->GetStartOutputDirectory());
+                              cmLocalGenerator::START_OUTPUT);
         this->WriteMakeRule(ruleFileStream,
                             "Manual pre-install relink rule for target.",
                             localName.c_str(), depends, commands, true);
@@ -697,13 +697,13 @@ cmLocalUnixMakefileGenerator3
     << " -E remove -f\n"
     << "\n";
   
-  if(this->Makefile->GetDefinition("CMAKE_EDIT_COMMAND"))
+  if(const char* edit_cmd =
+     this->Makefile->GetDefinition("CMAKE_EDIT_COMMAND"))
     {
     makefileStream
       << "# The program to use to edit the cache.\n"
       << "CMAKE_EDIT_COMMAND = "
-      << (this->ConvertToOutputForExisting(
-            this->Makefile->GetDefinition("CMAKE_EDIT_COMMAND"))) << "\n"
+      << this->Convert(edit_cmd,FULL,SHELL) << "\n"
       << "\n";
     }
 
@@ -835,7 +835,7 @@ void cmLocalUnixMakefileGenerator3
     {
     this->CreateCDCommand(commands,
                           this->Makefile->GetHomeOutputDirectory(),
-                          this->Makefile->GetStartOutputDirectory());
+                          cmLocalGenerator::START_OUTPUT);
     }
   this->WriteMakeRule(makefileStream,
                       "Special rule to run CMake to check the build system "
@@ -953,12 +953,13 @@ cmLocalUnixMakefileGenerator3
 void
 cmLocalUnixMakefileGenerator3
 ::AppendCustomCommands(std::vector<std::string>& commands,
-                       const std::vector<cmCustomCommand>& ccs)
+                       const std::vector<cmCustomCommand>& ccs,
+                       cmLocalGenerator::RelativeRoot relative)
 {
   for(std::vector<cmCustomCommand>::const_iterator i = ccs.begin();
       i != ccs.end(); ++i)
     {
-    this->AppendCustomCommand(commands, *i, true);
+    this->AppendCustomCommand(commands, *i, true, relative);
     }
 }
 
@@ -966,7 +967,8 @@ cmLocalUnixMakefileGenerator3
 void
 cmLocalUnixMakefileGenerator3
 ::AppendCustomCommand(std::vector<std::string>& commands,
-                      const cmCustomCommand& cc, bool echo_comment)
+                      const cmCustomCommand& cc, bool echo_comment,
+                      cmLocalGenerator::RelativeRoot relative)
 {
   // Optionally create a command to display the custom command's
   // comment text.  This is used for pre-build, pre-link, and
@@ -1072,8 +1074,7 @@ cmLocalUnixMakefileGenerator3
     }
 
   // Setup the proper working directory for the commands.
-  this->CreateCDCommand(commands1, dir,
-                        this->Makefile->GetHomeOutputDirectory());
+  this->CreateCDCommand(commands1, dir, relative);
 
   // push back the custom commands
   commands.insert(commands.end(), commands1.begin(), commands1.end());
@@ -1110,8 +1111,8 @@ cmLocalUnixMakefileGenerator3
     for(std::vector<std::string>::const_iterator f = files.begin();
         f != files.end(); ++f)
       {
-      fout << "\"" << this->Convert(f->c_str(),START_OUTPUT,UNCHANGED) 
-           << "\"\n";
+      std::string fc = this->Convert(f->c_str(),START_OUTPUT,UNCHANGED);
+      fout << "  " << this->EscapeForCMake(fc.c_str()) << "\n";
       }
     fout << ")\n";
     commands.push_back(remove);
@@ -1437,9 +1438,9 @@ cmLocalUnixMakefileGenerator3
 
   // create the file stream for the depends file
   std::string dir = targetDir;
-  
-  // Open the rule file.  This should be copy-if-different because the
-  // rules may depend on this file itself.
+
+  // Open the make depends file.  This should be copy-if-different
+  // because the make tool may try to reload it needlessly otherwise.
   std::string ruleFileNameFull = dir;
   ruleFileNameFull += "/depend.make";
   cmGeneratedFileStream ruleFileStream(ruleFileNameFull.c_str());
@@ -1448,11 +1449,14 @@ cmLocalUnixMakefileGenerator3
     {
     return false;
     }
+
+  // Open the cmake dependency tracking file.  This should not be
+  // copy-if-different because dependencies are re-scanned when it is
+  // older than the DependInfo.cmake.
   std::string internalRuleFileNameFull = dir;
   internalRuleFileNameFull += "/depend.internal";
-  cmGeneratedFileStream 
+  cmGeneratedFileStream
     internalRuleFileStream(internalRuleFileNameFull.c_str());
-  internalRuleFileStream.SetCopyIfDifferent(true);
   if(!internalRuleFileStream)
     {
     return false;
@@ -1470,68 +1474,18 @@ cmLocalUnixMakefileGenerator3
     {
     // construct the checker
     std::string lang = li->c_str();
-    
-    // Get the set of include directories.
-    std::vector<std::string> includes;
-    if(haveDirectoryInfo)
-      {
-      std::string includePathVar = "CMAKE_";
-      includePathVar += lang;
-      includePathVar += "_INCLUDE_PATH";
-      if(const char* includePath = mf->GetDefinition(includePathVar.c_str()))
-        {
-        cmSystemTools::ExpandListArgument(includePath, includes);
-        }
-      }
-    
-    // Get the include file regular expression.
-    std::string includeRegexScan = "^.*$";
-    std::string includeRegexComplain = "^$";
-    if(haveDirectoryInfo)
-      {
-      std::string scanRegexVar = "CMAKE_";
-      scanRegexVar += lang;
-      scanRegexVar += "_INCLUDE_REGEX_SCAN";
-      if(const char* scanRegex = mf->GetDefinition(scanRegexVar.c_str()))
-        {
-        includeRegexScan = scanRegex;
-        }
-      std::string complainRegexVar = "CMAKE_";
-      complainRegexVar += lang;
-      complainRegexVar += "_INCLUDE_REGEX_COMPLAIN";
-      if(const char* complainRegex = 
-         mf->GetDefinition(complainRegexVar.c_str()))
-        {
-        includeRegexComplain = complainRegex;
-        }
-      }
 
     // Create the scanner for this language
     cmDepends *scanner = 0;
     if(lang == "C" || lang == "CXX" || lang == "RC")
       {
-      std::string includeCacheFileName = dir;
-      includeCacheFileName += "/";
-      includeCacheFileName += lang;
-      includeCacheFileName += ".includecache";
-      
       // TODO: Handle RC (resource files) dependencies correctly.
-      scanner = new cmDependsC(includes,
-                               includeRegexScan.c_str(),
-                               includeRegexComplain.c_str(),
-                               includeCacheFileName);
+      scanner = new cmDependsC(this, targetDir, lang.c_str());
       }
 #ifdef CMAKE_BUILD_WITH_CMAKE
     else if(lang == "Fortran")
       {
-      std::vector<std::string> defines;
-      if(const char* c_defines =
-         mf->GetDefinition("CMAKE_TARGET_DEFINITIONS"))
-        {
-        cmSystemTools::ExpandListArgument(c_defines, defines);
-        }
-
-      scanner = new cmDependsFortran(includes, defines);
+      scanner = new cmDependsFortran(this);
       }
     else if(lang == "Java")
       {
@@ -1572,28 +1526,25 @@ void cmLocalUnixMakefileGenerator3::CheckMultipleOutputs(bool verbose)
   std::vector<std::string> pairs;
   cmSystemTools::ExpandListArgument(pairs_string, pairs, true);
   for(std::vector<std::string>::const_iterator i = pairs.begin();
-      i != pairs.end(); ++i)
+      i != pairs.end() && (i+1) != pairs.end();)
     {
-    const std::string& depender = *i;
-    if(++i != pairs.end())
-      {
-      const std::string& dependee = *i;
+    const std::string& depender = *i++;
+    const std::string& dependee = *i++;
 
-      // If the depender is missing then delete the dependee to make
-      // sure both will be regenerated.
-      if(cmSystemTools::FileExists(dependee.c_str()) &&
-         !cmSystemTools::FileExists(depender.c_str()))
+    // If the depender is missing then delete the dependee to make
+    // sure both will be regenerated.
+    if(cmSystemTools::FileExists(dependee.c_str()) &&
+       !cmSystemTools::FileExists(depender.c_str()))
+      {
+      if(verbose)
         {
-        if(verbose)
-          {
-          cmOStringStream msg;
-          msg << "Deleting primary custom command output \"" << dependee
-              << "\" because another output \""
-              << depender << "\" does not exist." << std::endl;
-          cmSystemTools::Stdout(msg.str().c_str());
-          }
-        cmSystemTools::RemoveFile(dependee.c_str());
+        cmOStringStream msg;
+        msg << "Deleting primary custom command output \"" << dependee
+            << "\" because another output \""
+            << depender << "\" does not exist." << std::endl;
+        cmSystemTools::Stdout(msg.str().c_str());
         }
+      cmSystemTools::RemoveFile(dependee.c_str());
       }
     }
 }
@@ -1660,9 +1611,11 @@ void cmLocalUnixMakefileGenerator3
       this->AppendCustomDepends(depends,   
                                 glIt->second.GetPostBuildCommands());
       this->AppendCustomCommands(commands, 
-                                 glIt->second.GetPreBuildCommands());
+                                 glIt->second.GetPreBuildCommands(),
+                                 cmLocalGenerator::START_OUTPUT);
       this->AppendCustomCommands(commands, 
-                                 glIt->second.GetPostBuildCommands());
+                                 glIt->second.GetPostBuildCommands(),
+                                 cmLocalGenerator::START_OUTPUT);
       std::string targetName = glIt->second.GetName();
       this->WriteMakeRule(ruleFileStream, targetString.c_str(), 
                           targetName.c_str(), depends, commands, true);
@@ -1724,7 +1677,7 @@ void cmLocalUnixMakefileGenerator3
                                                 recursiveTarget.c_str()));
   this->CreateCDCommand(commands,
                         this->Makefile->GetHomeOutputDirectory(),
-                        this->Makefile->GetStartOutputDirectory());
+                        cmLocalGenerator::START_OUTPUT);
     {
     cmOStringStream progCmd;
     progCmd << "$(CMAKE_COMMAND) -E cmake_progress_start "; // # 0
@@ -1746,7 +1699,7 @@ void cmLocalUnixMakefileGenerator3
                                                 recursiveTarget.c_str()));
   this->CreateCDCommand(commands,
                                 this->Makefile->GetHomeOutputDirectory(),
-                                this->Makefile->GetStartOutputDirectory());
+                                cmLocalGenerator::START_OUTPUT);
   this->WriteMakeRule(ruleFileStream, "The main clean target", "clean",
                       depends, commands, true);
   commands.clear();
@@ -1776,7 +1729,7 @@ void cmLocalUnixMakefileGenerator3
     (this->GetRecursiveMakeCall(mf2Dir.c_str(), recursiveTarget.c_str()));
   this->CreateCDCommand(commands,
                         this->Makefile->GetHomeOutputDirectory(),
-                        this->Makefile->GetStartOutputDirectory());
+                        cmLocalGenerator::START_OUTPUT);
   this->WriteMakeRule(ruleFileStream, "Prepare targets for installation.",
                       "preinstall", depends, commands, true);
   depends.clear();
@@ -1797,7 +1750,7 @@ void cmLocalUnixMakefileGenerator3
   commands.push_back(runRule);
   this->CreateCDCommand(commands,
                         this->Makefile->GetHomeOutputDirectory(),
-                        this->Makefile->GetStartOutputDirectory());
+                        cmLocalGenerator::START_OUTPUT);
   this->WriteMakeRule(ruleFileStream, "clear depends", 
                       "depend", 
                       depends, commands, true);
@@ -1925,6 +1878,32 @@ void cmLocalUnixMakefileGenerator3
     cmakefileStream
       << "  )\n";
     }
+
+  // Store include transform rule properties.  Write the directory
+  // rules first because they may be overridden by later target rules.
+  std::vector<std::string> transformRules;
+  if(const char* xform =
+     this->Makefile->GetProperty("IMPLICIT_DEPENDS_INCLUDE_TRANSFORM"))
+    {
+    cmSystemTools::ExpandListArgument(xform, transformRules);
+    }
+  if(const char* xform =
+     target.GetProperty("IMPLICIT_DEPENDS_INCLUDE_TRANSFORM"))
+    {
+    cmSystemTools::ExpandListArgument(xform, transformRules);
+    }
+  if(!transformRules.empty())
+    {
+    cmakefileStream
+      << "SET(CMAKE_INCLUDE_TRANSFORMS\n";
+    for(std::vector<std::string>::const_iterator tri = transformRules.begin();
+        tri != transformRules.end(); ++tri)
+      {
+      cmakefileStream << "  " << this->EscapeForCMake(tri->c_str()) << "\n";
+      }
+    cmakefileStream
+      << "  )\n";
+    }
 }
 
 //----------------------------------------------------------------------------
@@ -1949,12 +1928,12 @@ cmLocalUnixMakefileGenerator3
   obj += "/";
 
   // Get the object file name without the target directory.
-  std::string::size_type dir_len = 0;
-  dir_len += strlen(this->Makefile->GetCurrentOutputDirectory());
-  dir_len += 1;
-  dir_len += obj.size();
+  std::string dir_max;
+  dir_max += this->Makefile->GetCurrentOutputDirectory();
+  dir_max += "/";
+  dir_max += obj;
   std::string objectName =
-    this->GetObjectFileNameWithoutTarget(source, dir_len,
+    this->GetObjectFileNameWithoutTarget(source, dir_max,
                                          hasSourceExtension);
   if(nameWithoutTargetDir)
     {
@@ -2150,8 +2129,10 @@ cmLocalUnixMakefileGenerator3::AddImplicitDepends(cmTarget const& tgt,
 //----------------------------------------------------------------------------
 void cmLocalUnixMakefileGenerator3
 ::CreateCDCommand(std::vector<std::string>& commands, const char *tgtDir,
-                  const char *retDir)
+                  cmLocalGenerator::RelativeRoot relRetDir)
 {
+  const char* retDir = this->GetRelativeRootPath(relRetDir);
+
   // do we need to cd?
   if (!strcmp(tgtDir,retDir))
     {
@@ -2164,18 +2145,12 @@ void cmLocalUnixMakefileGenerator3
     // back because the shell keeps the working directory between
     // commands.
     std::string cmd = "cd ";
-    cmd += this->ConvertToOutputForExisting(tgtDir);
+    cmd += this->ConvertToOutputForExisting(tgtDir, relRetDir);
     commands.insert(commands.begin(),cmd);
-    
-    // Change back to the starting directory.  Any trailing slash must be
-    // removed to avoid problems with Borland Make.
-    std::string back = retDir;
-    if(back.size() && back[back.size()-1] == '/')
-      {
-      back = back.substr(0, back.size()-1);
-      }
+
+    // Change back to the starting directory.
     cmd = "cd ";
-    cmd += this->ConvertToOutputForExisting(back.c_str());
+    cmd += this->ConvertToOutputForExisting(relRetDir, tgtDir);
     commands.push_back(cmd);
     }
   else
@@ -2187,7 +2162,7 @@ void cmLocalUnixMakefileGenerator3
     for (; i != commands.end(); ++i)
       {
       std::string cmd = "cd ";
-      cmd += this->ConvertToOutputForExisting(tgtDir);
+      cmd += this->ConvertToOutputForExisting(tgtDir, relRetDir);
       cmd += " && ";
       cmd += *i;
       *i = cmd;
