@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmSourceFile.cxx,v $
   Language:  C++
-  Date:      $Date: 2008-05-01 16:35:40 $
-  Version:   $Revision: 1.47.2.2 $
+  Date:      $Date: 2008-10-24 15:18:54 $
+  Version:   $Revision: 1.47.2.4 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -55,7 +55,13 @@ const char* cmSourceFile::GetLanguage()
   // Perform computation needed to get the language if necessary.
   if(this->FullPath.empty() && this->Language.empty())
     {
-    if(this->Location.ExtensionIsAmbiguous())
+    // If a known extension is given or a known full path is given
+    // then trust that the current extension is sufficient to
+    // determine the language.  This will fail only if the user
+    // specifies a full path to the source but leaves off the
+    // extension, which is kind of weird.
+    if(this->Location.ExtensionIsAmbiguous() &&
+       this->Location.DirectoryIsAmbiguous())
       {
       // Finalize the file location to get the extension and set the
       // language.
@@ -185,16 +191,6 @@ bool cmSourceFile::FindFullPath()
       }
     }
 
-  // If the user provided a full path, trust it.  If the file is not
-  // there the native tool will complain at build time.
-  if(!this->Location.DirectoryIsAmbiguous())
-    {
-    this->FullPath = this->Location.GetDirectory();
-    this->FullPath += "/";
-    this->FullPath += this->Location.GetName();
-    return true;
-    }
-
   cmOStringStream e;
   e << "Cannot find source file \"" << this->Location.GetName() << "\"";
   e << ".  Tried extensions";
@@ -511,10 +507,22 @@ void cmSourceFile::DefineProperties(cmake *cm)
      "or Resources directories.");
 
   cm->DefineProperty
-    ("OBJECT_DEPENDS", cmProperty::SOURCE_FILE, 
-     "Additional dependencies.",
-     "Additional dependencies that should be checked as part of "
-     "building this source file.");
+    ("OBJECT_DEPENDS", cmProperty::SOURCE_FILE,
+     "Additional files on which a compiled object file depends.",
+     "Specifies a semicolon-separated list of full-paths to files on which "
+     "any object files compiled from this source file depend.  "
+     "An object file will be recompiled if any of the named files is newer "
+     "than it.\n"
+     "This property need not be used to specify the dependency of a "
+     "source file on a generated header file that it includes.  "
+     "Although the property was originally introduced for this purpose, it "
+     "is no longer necessary.  "
+     "If the generated header file is created by a custom command in the "
+     "same target as the source file, the automatic dependency scanning "
+     "process will recognize the dependency.  "
+     "If the generated header file is created by another target, an "
+     "inter-target dependency should be created with the add_dependencies "
+     "command (if one does not already exist due to linking relationships).");
 
   cm->DefineProperty
     ("OBJECT_OUTPUTS", cmProperty::SOURCE_FILE, 
