@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmWhileCommand.cxx,v $
   Language:  C++
-  Date:      $Date: 2008-03-01 02:33:33 $
-  Version:   $Revision: 1.12 $
+  Date:      $Date: 2009-02-04 16:44:17 $
+  Version:   $Revision: 1.12.2.2 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -21,13 +21,6 @@ bool cmWhileFunctionBlocker::
 IsFunctionBlocked(const cmListFileFunction& lff, cmMakefile &mf,
                   cmExecutionStatus &inStatus)
 {
-  // Prevent recusion and don't let this blocker block its own
-  // commands.
-  if (this->Executing)
-    {
-    return false;
-    }
-  
   // at end of for each execute recorded commands
   if (!cmSystemTools::Strucmp(lff.Name.c_str(),"while"))
     {
@@ -39,14 +32,18 @@ IsFunctionBlocked(const cmListFileFunction& lff, cmMakefile &mf,
     // if this is the endwhile for this while loop then execute
     if (!this->Depth) 
       {
-      char* errorString = 0;
+      // Remove the function blocker for this scope or bail.
+      cmsys::auto_ptr<cmFunctionBlocker>
+        fb(mf.RemoveFunctionBlocker(this, lff));
+      if(!fb.get()) { return false; }
+
+      std::string errorString;
     
       std::vector<std::string> expandedArguments;
       mf.ExpandArguments(this->Args, expandedArguments);
       bool isTrue = 
-        cmIfCommand::IsTrue(expandedArguments,&errorString,&mf);
+        cmIfCommand::IsTrue(expandedArguments,errorString,&mf);
 
-      this->Executing = true;
       while (isTrue)
         {      
         // Invoke all the functions that were collected in the block.
@@ -57,21 +54,18 @@ IsFunctionBlocked(const cmListFileFunction& lff, cmMakefile &mf,
           if (status.GetReturnInvoked())
             {
             inStatus.SetReturnInvoked(true);
-            mf.RemoveFunctionBlocker(lff);
             return true;
             }
           if (status.GetBreakInvoked())
             {
-            mf.RemoveFunctionBlocker(lff);
             return true;
             }
           }
         expandedArguments.clear();
         mf.ExpandArguments(this->Args, expandedArguments);
         isTrue = 
-          cmIfCommand::IsTrue(expandedArguments,&errorString,&mf);
+          cmIfCommand::IsTrue(expandedArguments,errorString,&mf);
         }
-      mf.RemoveFunctionBlocker(lff);
       return true;
       }
     else
@@ -104,15 +98,6 @@ ShouldRemove(const cmListFileFunction& lff, cmMakefile& )
   return false;
 }
 
-void cmWhileFunctionBlocker::
-ScopeEnded(cmMakefile &mf) 
-{
-  cmSystemTools::Error(
-    "The end of a CMakeLists file was reached with a WHILE statement that "
-    "was not closed properly. Within the directory: ", 
-    mf.GetCurrentDirectory());
-}
-
 bool cmWhileCommand
 ::InvokeInitialPass(const std::vector<cmListFileArgument>& args, 
                     cmExecutionStatus &)
