@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmWhileCommand.h,v $
   Language:  C++
-  Date:      $Date: 2006/06/30 17:48:46 $
-  Version:   $Revision: 1.4.2.1 $
+  Date:      $Date: 2009-02-04 16:44:18 $
+  Version:   $Revision: 1.9.2.1 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -29,16 +29,15 @@
 class cmWhileFunctionBlocker : public cmFunctionBlocker
 {
 public:
-  cmWhileFunctionBlocker() {Executing = false; Depth=0;}
+  cmWhileFunctionBlocker() {this->Depth=0;}
   virtual ~cmWhileFunctionBlocker() {}
   virtual bool IsFunctionBlocked(const cmListFileFunction& lff,
-                                 cmMakefile &mf);
+                                 cmMakefile &mf,
+                                 cmExecutionStatus &);
   virtual bool ShouldRemove(const cmListFileFunction& lff, cmMakefile &mf);
-  virtual void ScopeEnded(cmMakefile &mf);
   
   std::vector<cmListFileArgument> Args;
   std::vector<cmListFileFunction> Functions;
-  bool Executing;
 private:
   int Depth;
 };
@@ -63,13 +62,15 @@ public:
    * This overrides the default InvokeInitialPass implementation.
    * It records the arguments before expansion.
    */
-  virtual bool InvokeInitialPass(const std::vector<cmListFileArgument>& args);
+  virtual bool InvokeInitialPass(const std::vector<cmListFileArgument>& args,
+                                 cmExecutionStatus &);
     
   /**
    * This is called when the command is first encountered in
    * the CMakeLists.txt file.
    */
-  virtual bool InitialPass(std::vector<std::string> const&) { return false; }
+  virtual bool InitialPass(std::vector<std::string> const&,
+                           cmExecutionStatus &) { return false; }
 
   /**
    * This determines if the command is invoked when in script mode.
@@ -79,7 +80,7 @@ public:
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() { return "WHILE";}
+  virtual const char* GetName() { return "while";}
 
   /**
    * Succinct documentation.
@@ -95,16 +96,16 @@ public:
   virtual const char* GetFullDocumentation()
     {
     return
-      "  WHILE(condition)\n"
+      "  while(condition)\n"
       "    COMMAND1(ARGS ...)\n"
       "    COMMAND2(ARGS ...)\n"
       "    ...\n"
-      "  ENDWHILE(condition)\n"
-      "All commands between WHILE and the matching ENDWHILE are recorded "
-      "without being invoked.  Once the ENDWHILE is evaluated, the "
+      "  endwhile(condition)\n"
+      "All commands between while and the matching endwhile are recorded "
+      "without being invoked.  Once the endwhile is evaluated, the "
       "recorded list of commands is invoked as long as the condition "
-      "is true. The condition is evaulated using the same logic as the "
-      "IF command.";
+      "is true. The condition is evaluated using the same logic as the "
+      "if command.";
     }
   
   cmTypeMacro(cmWhileCommand, cmCommand);
