@@ -30,6 +30,21 @@
 # include <unistd.h>
 #endif
 
+#if defined(__BORLANDC__)
+# pragma warn -8060 /* possibly incorrect assignment */
+#endif
+
+#if defined(__BEOS__) && !defined(__ZETA__) && !defined(__HAIKU__)
+/* BeOS 5 doesn't have usleep(), but it has snooze(), which is identical. */
+# include <be/kernel/OS.h>
+static inline void testProcess_usleep(unsigned int msec)
+{
+  snooze(msec);
+}
+#else
+# define testProcess_usleep usleep
+#endif
+
 int runChild(const char* cmd[], int state, int exception, int value,
              int share, int output, int delay, double timeout, int poll,
              int repeat, int disown);
@@ -72,6 +87,9 @@ int test4(int argc, const char* argv[])
 #if defined(_WIN32)
   /* Avoid error diagnostic popups since we are crashing on purpose.  */
   SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX);
+#elif defined(__BEOS__) || defined(__HAIKU__)
+  /* Avoid error diagnostic popups since we are crashing on purpose.  */
+  disable_debugger(1);
 #endif
   (void)argc; (void)argv;
   fprintf(stdout, "Output before crash on stdout from crash test.\n");
@@ -196,7 +214,6 @@ int test8_grandchild(int argc, const char* argv[])
   return 0;
 }
 
-
 int runChild2(kwsysProcess* kp,
               const char* cmd[], int state, int exception, int value,
               int share, int output, int delay, double timeout,
@@ -261,7 +278,7 @@ int runChild2(kwsysProcess* kp,
 #if defined(_WIN32)
         Sleep(100);
 #else
-        usleep(100000);
+        testProcess_usleep(100000);
 #endif
         }
       if(delay)
@@ -273,14 +290,14 @@ int runChild2(kwsysProcess* kp,
         }
       }
     }
-  
+
   if(disown)
     {
     kwsysProcess_Disown(kp);
     }
   else
     {
-  kwsysProcess_WaitForExit(kp, 0);
+    kwsysProcess_WaitForExit(kp, 0);
     }
 
   switch (kwsysProcess_GetState(kp))
@@ -440,7 +457,7 @@ int main(int argc, const char* argv[])
     int values[8] = {0, 123, 1, 1, 0, 0, 0, 0};
     int outputs[8] = {1, 1, 1, 1, 1, 0, 1, 1};
     int delays[8] = {0, 0, 0, 0, 0, 1, 0, 0};
-    double timeouts[8] = {10, 10, 10, 10, 30, 10, -1, 10};
+    double timeouts[8] = {10, 10, 10, 30, 30, 10, -1, 10};
     int polls[8] = {0, 0, 0, 0, 0, 0, 1, 0};
     int repeat[8] = {2, 1, 1, 1, 1, 1, 1, 1};
     int r;
